@@ -23,6 +23,7 @@ default_style = {
     "silk": "#f0f0f0",
     "pads": "#b5ae30",
     "outline": "#000000",
+    "clad": "#9c6b28",
     "vcut": "#bf2600",
     "highlight-on-top": False,
     "highlight-style": "stroke:none;fill:#ff0000;opacity:0.5;",
@@ -245,8 +246,8 @@ def empty_svg(**attrs):
             "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">
         <svg xmlns="http://www.w3.org/2000/svg" version="1.1"
             width="29.7002cm" height="21.0007cm" viewBox="0 0 116930 82680 ">
-            <title>Picture generated by pcb2svg</title>
-            <desc>Picture generated by pcb2svg</desc>
+            <title>Picture generated by PcbDraw </title>
+            <desc>Picture generated by PcbDraw</desc>
         </svg>"""))
     root = document.getroot()
     for key, value in attrs.items():
@@ -301,16 +302,18 @@ def get_board_polygon(svg_elements):
                 e = etree.Element("path", d=path, style="fill-rule=evenodd;")
                 return e
 
-def process_board_substrate_layer(container, name, source, colors):
+def process_board_substrate_layer(container, name, source, colors, boardsize):
     layer = etree.SubElement(container, "g", id="substrate-"+name,
         style="fill:{0}; stroke:{0};".format(colors[name]))
     if name == "pads":
         layer.attrib["mask"] = "url(#pads-mask)"
+    if name == "silk":
+        layer.attrib["mask"] = "url(#pads-mask-silkscreen)"
     for element in extract_svg_content(read_svg_unique(source)):
         strip_fill_svg(element)
         layer.append(element)
 
-def process_board_substrate_base(container, name, source, colors):
+def process_board_substrate_base(container, name, source, colors, boardsize):
     clipPath = etree.SubElement(etree.SubElement(container, "defs"), "clipPath")
     clipPath.attrib["id"] = "cut-off"
     clipPath.append(get_board_polygon(extract_svg_content(read_svg_unique(source))))
@@ -324,15 +327,27 @@ def process_board_substrate_base(container, name, source, colors):
         strip_fill_svg(element)
         outline.append(element)
 
-def process_board_substrate_mask(container, name, source, colors):
+def process_board_substrate_mask(container, name, source, colors, boardsize):
     mask = etree.SubElement(etree.SubElement(container, "defs"), "mask")
     mask.attrib["id"] = name
     for element in extract_svg_content(read_svg_unique(source)):
         for item in element.getiterator():
             if "style" in item.attrib:
                 # KiCAD plots in black, for mask we need white
-                item.attrib["style"] = item.attrib["style"].replace("#000000", "#ffffff");
-            mask.append(element)
+                item.attrib["style"] = item.attrib["style"].replace("#000000", "#ffffff")
+        mask.append(element)
+    silkMask = etree.SubElement(etree.SubElement(container, "defs"), "mask")
+    silkMask.attrib["id"] = name + "-silkscreen"
+    for element in extract_svg_content(read_svg_unique(source)):
+        # KiCAD plots black, no need to change fill
+        silkMask.append(element)
+    bg = etree.SubElement(silkMask, "rect", attrib={
+        "x": str(ki2dmil(boardsize.GetX())),
+        "y": str(ki2dmil(boardsize.GetY())),
+        "width": str(ki2dmil(boardsize.GetWidth())),
+        "height": str(ki2dmil(boardsize.GetHeight())),
+        "fill": "white"
+    })
 
 def get_layers(board, colors, toPlot):
     """
@@ -345,6 +360,7 @@ def get_layers(board, colors, toPlot):
     popt.SetOutputDirectory(tmp)
     popt.SetScale(1)
     popt.SetMirror(False)
+    popt.SetSubtractMaskFromSilk(True)
     try:
         popt.SetPlotOutlineMode(False)
     except:
@@ -358,10 +374,11 @@ def get_layers(board, colors, toPlot):
             pctl.SetLayer(l)
             pctl.PlotLayer()
     pctl.ClosePlot()
+    boardsize = board.ComputeBoundingBox()
     for f, _, process in toPlot:
         for svg_file in os.listdir(tmp):
             if svg_file.endswith("-" + f + ".svg"):
-                process(container, f, os.path.join(tmp, svg_file), colors)
+                process(container, f, os.path.join(tmp, svg_file), colors, boardsize)
     shutil.rmtree(tmp)
     return container
 
@@ -374,6 +391,7 @@ def get_board_substrate(board, colors, holes, back):
     if(back):
         toPlot = [
             ("board", [pcbnew.Edge_Cuts], process_board_substrate_base),
+            ("clad", [pcbnew.B_Mask], process_board_substrate_layer),
             ("copper", [pcbnew.B_Cu], process_board_substrate_layer),
             ("pads", [pcbnew.B_Cu], process_board_substrate_layer),
             ("pads-mask", [pcbnew.B_Mask], process_board_substrate_mask),
@@ -382,6 +400,7 @@ def get_board_substrate(board, colors, holes, back):
     else:
         toPlot = [
             ("board", [pcbnew.Edge_Cuts], process_board_substrate_base),
+            ("clad", [pcbnew.F_Mask], process_board_substrate_layer),
             ("copper", [pcbnew.F_Cu], process_board_substrate_layer),
             ("pads", [pcbnew.F_Cu], process_board_substrate_layer),
             ("pads-mask", [pcbnew.F_Mask], process_board_substrate_mask),
@@ -395,6 +414,7 @@ def get_board_substrate(board, colors, holes, back):
     popt.SetOutputDirectory(tmp)
     popt.SetScale(1)
     popt.SetMirror(False)
+    popt.SetSubtractMaskFromSilk(True)
     try:
         popt.SetPlotOutlineMode(False)
     except:
@@ -408,10 +428,11 @@ def get_board_substrate(board, colors, holes, back):
             pctl.SetLayer(l)
             pctl.PlotLayer()
     pctl.ClosePlot()
+    boardsize = board.ComputeBoundingBox()
     for f, _, process in toPlot:
         for svg_file in os.listdir(tmp):
             if svg_file.endswith("-" + f + ".svg"):
-                process(container, f, os.path.join(tmp, svg_file), colors)
+                process(container, f, os.path.join(tmp, svg_file), colors, boardsize)
     shutil.rmtree(tmp)
 
     if holes:
@@ -466,9 +487,9 @@ def get_hole_mask(board):
         except:
             # Newest nightly renames Pads to PadList
             pad = module.PadsList()
-        orient = module.GetOrientation()
         while pad:
             pos = pad.GetPosition()
+            padOrientation = pad.GetOrientation()
             pos.x = ki2dmil(pos.x)
             pos.y = ki2dmil(pos.y)
             size = list(map(ki2dmil, pad.GetDrillSize()))
@@ -487,7 +508,7 @@ def get_hole_mask(board):
                 el.attrib["stroke-width"] = str(stroke)
                 el.attrib["points"] = points
                 el.attrib["transform"] = "translate({} {}) rotate({})".format(
-                    pos.x, pos.y, -orient)
+                    pos.x, pos.y, -padOrientation / 10)
             pad = pad.Next()
         module = module.Next()
     return defs
@@ -579,7 +600,7 @@ def load_style(style_file):
             style = json.load(f)
     except IOError:
         raise RuntimeError("Cannot open style " + style_file)
-    required = set(["copper", "board", "silk", "pads", "outline",
+    required = set(["copper", "board", "clad", "silk", "pads", "outline",
         "highlight-style", "highlight-offset", "highlight-on-top"])
     missing = required - set(style.keys())
     if missing:
@@ -640,7 +661,7 @@ def main():
             style = default_style
         remapping = load_remapping(args.remap)
     except RuntimeError as e:
-        print(e.message)
+        print(e)
         sys.exit(1)
 
     if os.path.splitext(args.output)[-1].lower() not in [".svg", ".png"]:
@@ -704,8 +725,8 @@ def main():
     walk_components(board, args.back, lambda lib, name, val, ref, pos:
         component_from_library(lib, name, val, ref, pos, components, highlight, args.silent))
 
-    #make another pass for search, and if found, render the back side of the component
-    #the function will search for file with extension ".back.svg"
+    # make another pass for search, and if found, render the back side of the component
+    # the function will search for file with extension ".back.svg"
     walk_components(board, not args.back, lambda lib, name, val, ref, pos:
         component_from_library(lib, name+".back", val, ref, pos, components, highlight, args.silent))
 
